#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <fcntl.h>
#include <errno.h>
#include <ctype.h>

#include "ppmvideoclip.h"
#include "colorspace.h"

// ---- bttv stuff -----
#include <asm/types.h>        /* some warnings with glibc */ 

#ifdef HAVE_CONFIG_H
#include <config.h>  // config.h by configure
#endif

#ifdef USE_KERNEL_VIDEODEV
#include <linux/videodev.h>
#else
#include <videodev.h>
#endif

static char graberror[256];
#define GRAB_PERROR(action,file) \
  {sprintf(graberror,"%s: %s: %s",action,file,strerror(errno));fprintf(stderr,graberror);return -1;}

#define TRAP(txt) fprintf(stderr,"%s:%d:%s\n",__FILE__,__LINE__,txt);exit(1);

/* ----------------------------------------------------------------------- */
ppmVideoClip::ppmVideoClip():videoClip()
{
  strcpy(ext,".ppm");
  strcpy(name,"PPM files");
  video_formats=VIDEO_PALETTE_RGB555 | VIDEO_PALETTE_RGB565 | VIDEO_PALETTE_RGB24 | VIDEO_PALETTE_RGB32;
  audio_formats=AUDIO_NONE;
}

int ppmVideoClip::open(char *filename, struct MOVIE_PARAMS *par)
{
  frames=0;
  strcpy(name,filename);strcat(name,"00000");if (strstr(name,ext) == NULL) strcat(name,ext);
  strcpy(par->extension,ext);
  memcpy(&params,par,sizeof(params));
  framebuf = (unsigned char *)malloc(params.width * params.height * 3);
  return 0;
}

void flip(unsigned char *mem, int len)
{
  int i;
  char tmp;

  for (i=0;i<len;i++) {
    tmp=mem[2];
    mem[2]=mem[0];
    mem[0]=tmp;
    mem+=3;
  }
}

int ppmVideoClip::writeframe(void *data)
{
    char header[80];
    int fh,rc=0;

    if (-1 == (fh = ::open(name,O_CREAT | O_RDWR | O_TRUNC, 0666)))
	GRAB_PERROR("open",name);
    patch_up(name);

#if 0
    if (params.video_format == VIDEO_GRAY8) {
	sprintf(header,"P5\n%d %d\n255\n",params.width,params.height);
	if (-1 == write(fh,header,strlen(header)))
	    GRAB_PERROR("open",name);
	if (-1 == write(fh,data,params.width*params.height))
	    GRAB_PERROR("write",name);
    } else {
#endif
	sprintf(header,"P6\n%d %d\n255\n",params.width,params.height);
	if (-1 == write(fh,header,strlen(header)))
	    GRAB_PERROR("open",name);
	switch (params.video_format) {
	case VIDEO_PALETTE_RGB555:
	    rgb15_to_rgb24(framebuf,data,params.width*params.height,0);
	    //flip(framebuf,params.width*params.height);
	    rc = write(fh,framebuf,params.width*params.height*3);
	    break;
	case VIDEO_PALETTE_RGB565:
	    rgb16_to_rgb24(framebuf,data,params.width*params.height,0);
	    //flip(framebuf,params.width*params.height);
	    rc = write(fh,framebuf,params.width*params.height*3);
	    break;
	case VIDEO_PALETTE_RGB24:
	    //flip((unsigned char *)data,params.width*params.height*3);
	    rc = write(fh,data,params.width*params.height);
	    break;
	case VIDEO_PALETTE_RGB32:
	    rgb32_to_rgb24(framebuf,data,params.width*params.height,0);
	    //flip(framebuf,params.width*params.height);
	    rc = write(fh,framebuf,params.width*params.height*3);
	    break;
	default:
	    TRAP("unsupported video format");
	}
#if 0
    }
#endif
    if (-1 == rc)
	GRAB_PERROR("write",name);
    ::close(fh);

    frames++;

    return 0;
}

int ppmVideoClip::close()
{
    free(framebuf);
    return 0;
}

/*
int ppmVideoClip::single_image(char *filename, void *data,
		 int width, int height, int format)
{
    int rc;
    params.width        = width;
    params.height       = height;
    params.video_format = format;
    strcpy(name,filename);
    framebuf = malloc(params.width * params.height * 3);
    rc = write(data);
    free(framebuf);
    return rc;
}
*/

/* file name count up */
void ppmVideoClip::patch_up(char *name)
{
    char *ptr;

    for (ptr = name+strlen(name); ptr >= name; ptr--)
	if (isdigit(*ptr))
	    break;
    if (ptr < name)
	return;
    while (*ptr == '9' && ptr >= name)
	*(ptr--) = '0';
    if (ptr < name)
	return;
    if (isdigit(*ptr))
	(*ptr)++;
}

/* ----------------------------------------------------------------------- */
